from pathlib import Path

from django.conf import settings
from django.core.management.commands import startapp

from ._utils import write_file, newlines, indent


class Command(startapp.Command):
    def handle(self, *args, **options):
        APPLICATIONS_ROOT = Path(settings.BASE_DIR) / "apps"
        APP_ROOT = APPLICATIONS_ROOT / options.get("name")

        # Create the application root
        APP_ROOT.mkdir()
        (APP_ROOT / "__init__.py").touch()
        write_file(
            APP_ROOT,
            "apps.py",
            (
                "from django.apps import AppConfig",
                *newlines(2),
                "class CoreConfig(AppConfig):",
                indent(1, f"name = '{APP_ROOT.name}'"),
                indent(1, f"label = '{APP_ROOT.name}'"),
            )
        )
        write_file(
            APP_ROOT,
            "urls.py",
            (
                "from django.urls import path",
                "from rest_framework import routers",
                "from django.apps import apps",
                "from .apps import CoreConfig as CurrentApp",
                *newlines(),
                "app_name = CurrentApp.name",
                newlines(),
                "router = routers.SimpleRouter()",
                newlines(),
                "urlpatterns = []",
                newlines(),
                "urlpatterns += router.urls",
            )
        )
        write_file(
            APP_ROOT,
            "admin.py",
            (
                "from django.contrib import admin",
                *newlines(2),
                "# Register your models here"
            )
        )
        write_file(
            APP_ROOT,
            "views.py",
            (
                "from django.views import View",
                "from rest_framework import mixins, viewsets",
                *newlines(2),
                "# Your views here"
            )
        )

        # Create the migrations directory
        migrations = APP_ROOT / "migrations"
        migrations.mkdir()
        (migrations / "__init__.py").touch()

        # Create the schemas directory
        models = APP_ROOT / "models"
        models.mkdir()
        write_file(
            models,
            "__init__.py",
            (
                "from .models import *",
            )
        )
        write_file(
            models,
            "serializers.py",
            (
                "from rest_framework import serializers",
                *newlines(2),
                "# Your serializers here",
            )
        )
        write_file(
            models,
            "models.py",
            (
                "from django.db import models",
                *newlines(2),
                "# Your models here",
            )
        )

        # Create the templates directory
        templates = APP_ROOT / "templates"
        templates.mkdir()
        (templates / APP_ROOT.name).mkdir()



from django.contrib.admin import AdminSite


class SortableAdminMenuMixin(AdminSite):
    admin_app_ordering: dict

    @staticmethod
    def _merge_apps(toplevel_app: dict,
                    mergees: list) -> dict:
        new_app = toplevel_app
        for app in mergees:
            new_app["models"] += app.get("models")
        return new_app

    @staticmethod
    def get_app_index(app_list: list, app_label: str) -> int:
        for i, app in enumerate(app_list):
            if app.get("app_label") == app_label:
                return i

    def _assemble_app_dict(self,
                           app_list: list,
                           app_label: str) -> dict:
        original_app = app_list[
            self.get_app_index(app_list, app_label)
        ]
        additional_app_labels = self.admin_app_ordering.get(app_label, tuple())
        if additional_app_labels:
            additional_app_labels = tuple(
                i for i in additional_app_labels if not i == app_label
            )
        apps_to_merge = [
            app_list[
                self.get_app_index(app_list, a)
            ] for a in additional_app_labels
        ]
        return self._merge_apps(original_app, apps_to_merge)

    def get_app_list(self, request):
        """ Overrides the app_list based on the admin_app_ordering parameter """
        app_list = super(SortableAdminMenuMixin, self).get_app_list(request)
        # Do not process if the attribute has not been set
        if not self.admin_app_ordering:
            return app_list
        # If we are at the login site
        if request.user.id is None:
            return app_list

        new_app_list = []
        for original_app_label, additional_app_labels in self.admin_app_ordering.items():
            # This means we need to make changes to the app_dict
            if len(additional_app_labels) > 1:
                new_app_list.append(
                    self._assemble_app_dict(
                        app_list, original_app_label
                    )
                )

            # In this case the app_dict is fine as it is
            else:
                new_app_list.append(
                    app_list[
                        self.get_app_index(app_list, original_app_label)
                    ]
                )

        return new_app_list



from .fields import SelfHrefField
from rest_framework import serializers
from typing import Union, Iterable, Optional


class HrefModelSerializer(serializers.HyperlinkedModelSerializer):
    url_field_name = "self"
    serializer_url_field = SelfHrefField

    class Meta:
        fields: Union[str, Iterable]
        serializer_class: serializers.Serializer
        self_view: Optional[str] = None


__all__ = ["HrefModelSerializer"]



from rest_framework.mixins import CreateModelMixin


class DetailSerializerMixin:
    serializer_detail_class = None
    queryset_detail = None

    def get_serializer_class(self):
        error_message = "" \
                        "'{0}' should include a " \
                        "'serializer_detail_class' attribute".format(
            self.__class__.__name__)
        assert self.serializer_detail_class is not None, error_message
        if self.request_is_to_detail_endpoint():
            return self.serializer_detail_class
        else:
            return super().get_serializer_class()

    def get_queryset(self, *args, **kwargs):
        if self.request_is_to_detail_endpoint() and self.queryset_detail is not None:
            return self.queryset_detail.all()
        else:
            return super().get_queryset(*args, **kwargs)

    def request_is_to_detail_endpoint(self) -> bool:
        if hasattr(self, 'lookup_url_kwarg'):
            lookup = self.lookup_url_kwarg or self.lookup_field
        return lookup and lookup in self.kwargs


class OverwriteModelMixin(CreateModelMixin):
    def update(self, request, *args, **kwargs):
        return self.create(
            request,
            *args,
            **kwargs
        )


__all__ = ["DetailSerializerMixin", "OverwriteModelMixin"]



from typing import Optional, Union, Tuple
from pathlib import Path


def newlines(amount: Optional[int] = None) -> Union[Tuple[str], str]:
    return tuple(
        "" for _ in range(amount)
    ) if amount is not None else ""


def indent(indents: int, line_contents: str) -> str:
    return "{line_contents: >{width}}".format(
        line_contents=line_contents,
        width=len(line_contents) + (indents*4)
    )


def write_file(base_dir: Path,
               filename: str,
               content: Union[tuple, list]) -> None:
    with (base_dir / filename).open("w") as fout:
        for line in content:
            fout.write(f"{line}\n")



from typing import Dict, Any, Optional

from django.core.exceptions import ImproperlyConfigured
from django.urls import NoReverseMatch
from rest_framework import serializers
from rest_framework.relations import Hyperlink


class SelfHrefField(serializers.HyperlinkedIdentityField):
    def __init__(self, view_name=None, **kwargs):
        if not view_name and hasattr(self.parent, "Meta"):
            view_name = getattr(self.parent.Meta, "self_view")
        assert view_name is not None, 'The `view_name` argument is required.'
        kwargs['read_only'] = True
        kwargs['source'] = '*'
        super().__init__(view_name, **kwargs)

    def _get_url_representation(self,
                                value: Any,
                                view_name: Optional[str] = None
                                ) -> Optional[str]:
        assert "request" in self.context, (
                "`%s` requires the request in the serializer"
                " context. Add `context={'request': request}` when instantiating "
                "the serializer." % self.__class__.__name__
        )

        request = self.context["request"]
        format_ = self.context.get("format", None)

        if format_ and self.format and self.format != format_:
            format_ = self.format

        try:
            reverse_view = view_name or self.view_name
            url = self.get_url(value, reverse_view, request, format_)
        except NoReverseMatch:
            msg = (
                "Could not resolve URL for hyperlinked relationship using "
                "view name '%s'. You may have failed to include the related "
                "model in your API, or incorrectly configured the "
                "`lookup_field` attribute on this field."
            )
            if value in ("", None):
                value_string = {"": "the empty string", None: "None"}[value]
                msg += (
                        " WARNING: The value of the field on the model instance "
                        "was %s, which may be why it didn't match any "
                        "entries in your URL conf." % value_string
                )
            raise ImproperlyConfigured(msg % self.view_name)

        if url is None:
            return None

        return Hyperlink(url, value)

    def to_representation(self, value) -> Dict[str, str]:
        url = self._get_url_representation(
            value, self.view_name
        )
        return {"href": url}



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['drizm_django_commons',
 'drizm_django_commons.management.commands',
 'drizm_django_commons.serializers']

package_data = \
{'': ['*']}

install_requires = \
['django==3.1.1', 'djangorestframework==3.12.1']

setup_kwargs = {
    'name': 'drizm-django-commons',
    'version': '0.2.2',
    'description': 'Django commons for the Drizm organization',
    'long_description': '# Django Commons\n[![PyPI version](https://badge.fury.io/py/drizm-django-commons.svg)](https://badge.fury.io/py/drizm-django-commons)\n\nThis package includes shared code used by\nthe Drizm organizations development team.  \n\nIt is not intended for public usage but you\nmay still download, redistribute or \nmodify it to your liking.\n\n## Installation\n\nInstall:  \n>pip install drizm-django-commons\n\nOnce installed through pip, include\nthe app in your settings.py like so:  \nINSTALLED_APPS += ["drizm_django_commons"]  \n\nIn order to use the applications\nmanage.py commands you must include the\napp at the top of the INSTALLED_APPS list.\n\nImport like so:  \nimport drizm_django_commons\n\n## Documentation\n\npass\n\n## Changelog\n\n### 0.2.1\n\n- Added HrefModelSerializer which will\nserialize primary keys to hyperlinks\n- Moved testing.py dependencies to\ndrizm-commons package utilities\n\n### 0.2.2\n\n- Fixed bug with view selection for SelfHrefField\n',
    'author': 'ThaRising',
    'author_email': 'kochbe.ber@gmail.com',
    'maintainer': 'Dominik Lewandowski',
    'maintainer_email': 'dominik.lewandow@gmail.com',
    'url': 'https://github.com/drizm-team/django-commons',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from rest_framework.pagination import (
    LimitOffsetPagination,
    _positive_int  # noqa access to protected
)
from typing import ClassVar

from rest_framework.response import Response
from rest_framework import status


class MaxLimitOffsetPagination(LimitOffsetPagination):
    max_page_size: ClassVar[int] = 100

    def get_limit(self, request) -> int:
        if self.limit_query_param:
            try:
                param = request.query_params[self.limit_query_param]
            except KeyError:
                return self.default_limit

            try:
                amount = int(param)
            except ValueError:
                amount = param
                if type(amount) == str and amount == "max":
                    amount = self.max_limit
                else:
                    return self.default_limit
            return _positive_int(
                amount,
                strict=True,
                cutoff=self.max_limit
            )

    def get_paginated_response(self, data) -> Response:
        default = super(
            MaxLimitOffsetPagination, self
        ).get_paginated_response(data)
        default.status_code = status.HTTP_206_PARTIAL_CONTENT
        default.data["prev"] = default.data.pop("previous")
        return default

    def get_paginated_response_schema(self, schema) -> dict:
        default = super(
            MaxLimitOffsetPagination, self
        ).get_paginated_response_schema(schema)
        previous = default["properties"].pop("previous")
        default["properties"]["prev"] = previous
        return default
